#include "files.h"
#include "io.h"
#include "mainloop.h"

char fd_name[4][FNLENGTH];
char hd_name[2][FNLENGTH];
char lst_name[FNLENGTH];
char rdr_name[FNLENGTH];
char rom_name[FNLENGTH];
char pun_name[FNLENGTH];

FILE *pun_file = NULL;
FILE *rdr_file = NULL;
FILE *lst_file = NULL;

uint8_t noboot_prog[26] = {
	0x11, 0x10, 0x00, 0x1a, 0xf6, 0x00, 0xca, 0x0f,
	0x00, 0xd3, 0x01, 0x13, 0xc3, 0x03, 0x00, 0x76,
	'N', 'o', ' ', 'b', 'o', 'o', 't', 0x0d, 0x01,
	0x00
};

void init_filenames(void) {
	int i;
	for(i = 0; i < 4; i++) {
		fd_name[i][0] = 0;
	}
	for(i = 0; i < 2; i++) {
		hd_name[i][0] = 0;
	}
	rom_name[0] = 0;
	rdr_name[0] = 0;
	pun_name[0] = 0;
	lst_name[0] = 0;
	return;
}

void close_all_files(void) {
	if(pun_file) {
		fclose(pun_file);
	}
	if(rdr_file) {
		fclose(rdr_file);
	}
	if(lst_file) {
		fclose(lst_file);
	}
	return;
}

int32_t readrom(char *filename, int32_t start) {
	FILE *fp;
	int32_t i;
	int32_t c;

	if(!(fp = fopen(filename, "r"))) {
		fprintf(stderr, "Error: non-existing file - %s\n", filename);
		cleanexit(2);
	}
	for(i = 0; i < 65535 - start; i++) {
		c = getc(fp);
		if(feof(fp)) {
			break;
		}
		mem[i + start] = (uint8_t) c;
	}
	fclose(fp);
	return 0;
}

int32_t readdisk(char *filename, int32_t drive) {
	FILE *fp;
	int32_t i, j, k;
	int32_t c;

	if(!(fp = fopen(filename, "r"))) {
		printf("Creating new disk: %s\n", filename);
		for(i=0; i<77; i++) {
			for(j=0; j<26; j++) {
				for(k = 0; k < 128; k++) {
					disk[drive][i][j][k] = 0;
				}
			}
		}
		for(i=0; i<26; i++) {
			disk[drive][0][0][i] = noboot_prog[i];
		}
		return 0;
	}
	for(i=0; i<77; i++) {
		for(j=0; j<26; j++) {
			for(k = 0; k < 128; k++) {
				c = fgetc(fp);
				if(feof(fp)) {
					fprintf(stderr, "Error: Diskfile too short - %s\n",
						filename);
					break;
				}
				disk[drive][i][j][k] = (uint8_t) c;
			}
		}
	}
	fclose(fp);
	return 0;
}

int32_t writedisk(char *filename, int32_t drive) {
	FILE *fp;
	int32_t i, j, k;

	if(!(fp = fopen(filename, "w"))) {
		fprintf(stderr, "Error: writing error - %s\n", filename);
		cleanexit(2);
	}
	for(i=0; i<77; i++) {
		for(j=0; j<26; j++) {
			for(k = 0; k < 128; k++) {
				fputc((int) disk[drive][i][j][k], fp);
			}
		}
	}
	fclose(fp);
	return 0;
}

int32_t readhd(char *filename, int32_t drive) {
	FILE *fp;
	int32_t i, j, k;
	int32_t c;

	if(!(fp = fopen(filename, "r"))) {
		printf("Creating new hard disk: %s\n", filename);
		for(i=0; i<255; i++) {
			for(j=0; j<128; j++) {
				for(k = 0; k < 128; k++) {
					hd[drive][i][j][k] = 0;
				}
			}
		}
		return 0;
	}
	for(i=0; i<255; i++) {
		for(j=0; j<128; j++) {
			for(k = 0; k < 128; k++) {
				c = fgetc(fp);
				if(feof(fp)) {
					fprintf(stderr, "Error: Diskfile too short - %s\n",
						filename);
					break;
				}
				hd[drive][i][j][k] = (uint8_t) c;
			}
		}
	}
	fclose(fp);
	return 0;
}

int32_t writehd(char *filename, int32_t drive) {
	FILE *fp;
	int32_t i, j, k;

	if(!(fp = fopen(filename, "w"))) {
		fprintf(stderr, "Error: writing error - %s\n", filename);
		cleanexit(2);
	}
	for(i=0; i<255; i++) {
		for(j=0; j<128; j++) {
			for(k = 0; k < 128; k++) {
				fputc((int) hd[drive][i][j][k], fp);
			}
		}
	}
	fclose(fp);
	return 0;
}

