#include "io.h"
#include "term.h"
#include "files.h"
#include "mainloop.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <termios.h>

uint8_t disk[4][77][26][128];
uint8_t hd[2][255][128][128];
uint8_t altair = 0;
uint8_t swap_ru_bs = 0;
uint8_t mem[65536];

int32_t xgetc(int32_t i);
void xputc(uint8_t c);
void writetotape(uint8_t c);
void writetoprinter(uint8_t c);
int32_t readfromtape(void);
int32_t processescape(void);

uint8_t io_drive = 0;
uint8_t io_track = 0;
uint8_t io_sector = 0;
uint16_t io_dma;
uint8_t io_diskstatus;

uint8_t bdos_prog[62] = {
	0x76, 0x00, 0x00, 0x00, 0x00, 0x32, 0x01, 0x00,
	0x78, 0x32, 0x02, 0x00, 0xeb, 0x22, 0x03, 0x00,
	0xeb, 0x79, 0xfe, 0x02, 0xca, 0x1d, 0x00, 0xfe,
	0x09, 0xca, 0x25, 0x00, 0x76, 0x3a, 0x01, 0x00,
	0xd3, 0x01, 0xc3, 0x31, 0x00, 0x1a, 0xfe, 0x24,
	0xca, 0x31, 0x00, 0xd3, 0x01, 0x13, 0xc3, 0x25,
	0x00, 0x3a, 0x02, 0x00, 0x47, 0x3a, 0x01, 0x00,
	0xeb, 0x2a, 0x03, 0x00, 0xeb, 0xc9
};

void setaltair(void) {
	altair = 1;
}

void bdos_load(void) {
	int32_t i;

	for(i = 0; i < 62; i++) {
		write_mem(i, bdos_prog[i]);
	}
}

int32_t read_mem(uint16_t addr) {
	return mem[addr];
}

int32_t write_mem(uint16_t address, uint8_t value) {
	mem[address] = value;
	return 0;
}

int32_t processescape(void) {
	int32_t c = 0xff & xgetc(1);
	if(c == 27) {
		return 1000;
	}
	if(c == 'e') {
		return 27;
	}
	if(c == 'q') {
		cleanexit(0);
	}
	return c;
}

int32_t input(uint8_t port) {
	static int32_t c;
	static int32_t keyfetched = 0;

	switch(port) {
		case 0:
			if(keyfetched == 1) {
				c = xgetc(0);
				if(c == 27) {
					c = processescape();
					if(c == 1000) {
						return c;
					}
				}
				if(c == EOF) {
					return(altair ? 0x01 : 0x00);
				} else {
					keyfetched = 0;
					return(altair ? 0x00 : 0xff);
				}
			} else {
				return(altair ? 0x00 : 0xff);
			}
		case 1:
			if(keyfetched == 1) {
				c = EOF;
			}
			if(c == EOF) {
				c = xgetc(1);
				if(c == 27) {
					return processescape();
				}
			}
			keyfetched = 1;
			return(c == '\n' ? '\r' : c);
		case 4:
			return 0xff;
		case 5:
		case 7:
			return readfromtape();
		case 6:
			return 0x00;
		case 13:
			return 1;
		case 14:
			return io_diskstatus;
		case 255:
			return 0;
		default:
			printf("IN: port %d\n", port);
			return 0;
	}
} 

int32_t output(uint8_t port, uint8_t value) {
	int32_t i;

	switch(port) {
		case 1:
			xputc(value);
			return 0;
		case 3:
			writetoprinter(value);
			return 0;
		case 5:
		case 7:
			writetotape(value);
			return 0;
		case 10: // 0x0a FDC drive
			io_drive = value;
			break;
		case 11: // 0x0b FDC track
			io_track = value;
			break;
		case 12: // 0x0c FDC sector
			io_sector = value;
			break;
		case 13: // 0x0d FDC command
			if(io_drive > 5) { /* Num of disks */
				io_diskstatus = 2; // illegal drive
			} else if(io_drive > 3) {
				if(io_track >= 255) {
					io_diskstatus = 2;
				} else if(io_sector == 0 || io_sector > 128) {
					io_diskstatus = 3;
				} else if(value == 0) {
					if(io_dma >= 65536 - 128) {
						io_diskstatus = 5;  // read error
					} else {
						for(i = 0; i < 128; i++) {
							mem[io_dma + i] =
								hd[io_drive-4][io_track][io_sector-1][i];
						}
					}
				} else if(value == 1) {
					if(io_dma >= 65536 - 128) {
						io_diskstatus = 6;  // read error
					} else {
						for(i = 0; i < 128; i++) {
							hd[io_drive-4][io_track][io_sector-1][i] =
								mem[io_dma + i];
						}
					}
				} else {
					 io_diskstatus = 7;
				}
			} else if(io_track >= 77) { /* Num of tracks */
				io_diskstatus = 2; // illegal track
			} else if(io_sector == 0 || io_sector > 26) {
				io_diskstatus = 3; // illegal sector
			} else if(value == 0) {        // read
				if(io_dma >= 65536 - 128) {
					io_diskstatus = 5;  // read error
				} else {
					for(i = 0; i < 128; i++) {
						mem[io_dma + i] =
							disk[io_drive][io_track][io_sector-1][i];
					}
				}
			} else if(value == 1) { // write
				if(io_dma >= 65536 - 128) {
					io_diskstatus = 6;  // read error
				} else {
					for(i = 0; i < 128; i++) {
						disk[io_drive][io_track][io_sector-1][i] =
							mem[io_dma + i];
					}
				}
			} else {
				io_diskstatus = 7;    // illegal command
			}
			break;
		case 15: // 0x0f DMA low
			io_dma = (io_dma & 0xff00) | value;
			break;
		case 16: // 0x10 DMA high
			io_dma = (io_dma & 0x00ff) | ((uint16_t) value << 8);
			break;
		default:
			printf("OUT: port %d %d\n", port, value);
	}
	return 0;
}

void resetINTE(void) {
	return;
}

void setINTE(void) {
	return;
}

int32_t bootload(int32_t drive) {
	int32_t i;

	for(i = 0; i < 128; i++) {
		mem[i] = disk[0][0][0][i];
	}
	return 0;
}

int32_t xgetc(int32_t i) {
	uint8_t c;
	fflush(stdout);
	if(i == 1) {
		setcterm(1);
		if(read(STDIN_FILENO, &c, 1))
			;
		setcterm(0);
	}
	if(i == 0 && read(STDIN_FILENO, &c, 1) <= 0) {
		return EOF;
	} else if(c == 127) {
		if(altair) {
			return '_';
		} else if(swap_ru_bs) {
			return 127;
		} else {
			return 8;
		}
	} else if(c == 8) {
		if(swap_ru_bs) {
			return 8;
		} else {
			return 127;
		}
	} else if(altair && c >= 97 && c <= 122) {
		return c - 32;
	} else {
		return c;
	}
}

void xputc(uint8_t c) {
	if(altair) {
		if(c == '_') {
			printf("\b \b");
			fflush(stdout);
			return;
		}
	}
	if(c > 127) {
		c = c - 128;
	}
	if(write(STDOUT_FILENO, &c, 1))
		;
}

void writetotape(uint8_t c) {
	if(!pun_file) {
		if((pun_file = fopen(pun_name,"w")) == NULL) {
			return;
		}
	}
	if(fputc(c, pun_file))
		;
}

void writetoprinter(uint8_t c) {
	if(!lst_file) {
		if((lst_file = fopen(lst_name,"w")) == NULL) {
			return;
		}
	}
	if(fputc(c, lst_file))
		;
}

int32_t readfromtape(void) {
	int c;
	if(!rdr_file) {
		if((rdr_file = fopen(rdr_name,"r")) == NULL) {
			return altair ? 1000 : 26;
		}
	}
	c = fgetc(rdr_file);
	if(c==EOF) {
		return altair ? 1000 : 26;
	}
	return c;
}

