#include <time.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>

#include "io.h"
#include "8080.h"
#include "mainloop.h"
#include "files.h"
#include "term.h"

#define Xsleep(T) (usleep((T)/1000))
#define BILLION (1000000000)
#define MILLION (1000000)

double speed = 0; /* Speed in MHz, 0 = as fast as possible */
double refresh_rate = 60; /* Refresh rate in Hz */
long long nanosecs_per_iteration;
long cycles_per_iteration;

struct timespec tp;

unsigned long long cur_time();

int32_t mainloop(uint16_t start, double speed) {
	unsigned long long counter = cur_time();
	unsigned long long current_clock;

	int32_t quit_signal = 0;

	nanosecs_per_iteration = BILLION / refresh_rate;
	cycles_per_iteration = ((speed * MILLION) / refresh_rate);

	init_emulator(start);

	while(!quit_signal) {
		counter += nanosecs_per_iteration;
		quit_signal = run_emulator(cycles_per_iteration);
		if(counter > (current_clock = cur_time())) {
			Xsleep(counter - current_clock);
		} else if(!quit_signal) {
			counter = cur_time(); /* Running too slow... */
		}
	}
	return quit_signal;
}

void cleanexit(int status) {
	close_all_files();
	resetterm();
	printf("\n");
	exit(status);
}

unsigned long long cur_time() {
	clock_gettime(CLOCK_REALTIME, &tp);
	return (tp.tv_sec) * BILLION + (tp.tv_nsec);
}

